import { computed, Ref, ref, watch } from 'vue';
import { deepCloneJSON } from '@codeperate/utils';
import { FormBuilderBaseContext } from '.';
import { debounce } from '../utils';

export interface UseHistoryOptions {
    historyLength?: number;
    historyInterval?: number;
}

export interface UseHistory {
    history: Ref<any[]>;
    canUndo: Ref<boolean>;
    canRedo: Ref<boolean>;
    undo: () => void;
    redo: () => void;
}

export const useHistory = (
    { formValue, setValue }: FormBuilderBaseContext,
    { historyLength = 0, historyInterval = 2000 }: UseHistoryOptions = {},
) => {
    const history = ref<any[]>([]);
    const currentIndex = ref(-1);
    const canUndo = computed(() => currentIndex.value >= 0);
    const canRedo = computed(() => currentIndex.value < history.value.length - 1);
    const saveHistory = () => {
        if (JSON.stringify(history.value[currentIndex.value]) !== JSON.stringify(formValue.value)) {
            if (currentIndex.value < history.value.length - 1) {
                history.value = history.value.slice(0, currentIndex.value + 1);
            }
            history.value.push(deepCloneJSON(formValue.value));
            currentIndex.value = history.value.length - 1;
            if (history.value.length > historyLength) {
                history.value = history.value.slice(-historyLength);
                currentIndex.value = history.value.length - 1;
            }
        }
    };
    const debouncedSaveHistory = debounce(saveHistory, historyInterval);
    if (historyLength && historyLength > 0) {
        watch(
            () => formValue,
            () => debouncedSaveHistory(),
            { deep: true },
        );
    }

    const undo = () => {
        if (canUndo.value) {
            currentIndex.value--;
            setValue('', history.value[currentIndex.value]);
        }
    };

    const redo = () => {
        if (canRedo.value) {
            currentIndex.value++;
            setValue('', history.value[currentIndex.value]);
        }
    };

    return {
        history,
        canUndo,
        canRedo,
        undo,
        redo,
    };
};
